/*
 $Id: main.c,v 1.15 2019/07/07 10:34:45 daeb Exp $

 Init-part - main loop with command interpreter

*/
/*

 Copyright (C) 2019 Bernhard Dubler ( klappe@daeublers.de )

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
/*

Kommandos:
	"up" :	Riegel hoch
	"down" : Riegel runter
	"close" : 4 Wege Schalter auf "zu"
	"in" : 4 Wege Schalter auf "nur rein"
	"out" : 4 Wege Schalter auf "nur raus"
	"inout" : 4 Wege Schalter auf "auf"
	["-"]zahl : bewege 4 Wege Schalter um entsprechende Grad-zahl

Ergebnisse:
	ok: command			- quittierung
	unknown command
	id: ccc.iiiiiiiiiiii ["bad crc"]	- rfid gelesen, ggf bad crc
	motion				- motion sensor getriggert
	opened				- klappe ging auf - egal wie lang oder wie rum
	in Xs ago			- katze kam vor X sekunden rein
	out Xs ago			- katze ging vor X sekunden raus
	in open	Xs ago		- katze hat klappe vor X sekunden nach innen geffnet und >=10s offen gelassen
	in close Xs ago		- klappe vor X sekunden nach "in open" wieder zu
	out open Xs ago		- katze hat klappe vor X sekunden nach aussen geffnet und >=10s offen gelassen
	out close Xs ago	- klappe vor X sekunden nach "out open" wieder zu

*/
/*
  Summary:
    This is the main file generated using PIC10 / PIC12 / PIC16 / PIC18 MCUs

  Description:
    This header file provides implementations for driver APIs for all modules selected in the GUI.
    Generation Information :
        Product Revision  :  PIC10 / PIC12 / PIC16 / PIC18 MCUs - 1.65.2
        Device            :  PIC16F1459
        Driver Version    :  2.00
*/


#include <stdlib.h>
#include "mcc_generated_files/mcc.h"
#include "io.h"
#include "def.h"
#include "stepper.h"
#include "sensors.h"
#include "time.h"
#include "rfid.h"

#define CMDBUFFER_LEN 10
static char cmdbuffer[CMDBUFFER_LEN];

struct cmd
{	char *cmd_txt;
	int (*cmd_fn)(void);
};

#ifdef DEFTIME

static int uptime(void)
{		putstr("up: ");
		print_time();
		return 1;
}

#endif
#ifdef DEBUGRFID

static int debug_state(void)
{		print_rfid_state();
		return 0;
}

static int debug_start(void)
{		Sflag=1;
		putstr("\r\nSFlag="); print16(Sflag,0); putstr("\r\n");
		return 0;
}

static int debug_end(void)
{		Sflag=0;
		putstr("\r\nSFlag="); print16(Sflag,0); putstr("\r\n");
		return 0;
}

#endif

static int riegel_up(void)
{		move_riegel(1);
		return 0;
}

static int riegel_down(void)
{		move_riegel(0);
		return 0;
}

static int close(void)
{		move_stepper(-270);
		return 0;
}

static int in(void)
{		move_stepper(-270);
		move_stepper(80);
		return 0;
}

static int out(void)
{		move_stepper(270);
		move_stepper(-102);
		return 0;
}

static int inout(void)
{		move_stepper(270);
		move_stepper(-12);
		return 0;
}


struct cmd cmds[] = {
#ifdef DEBUGRFID
	{ "s", debug_state},
	{ "S", debug_start},
	{ "e", debug_end},
#endif
#ifdef DEFTIME
	{ "uptime", uptime},
#endif
	{ "up", riegel_up},
	{ "down", riegel_down},
	{ "close", close},
	{ "in", in},
	{ "out", out},
	{ "inout", inout} };

/*
                         Main application
 */
void main(void)
{
    // initialize the device
    SYSTEM_Initialize();

    // When using interrupts, you need to set the Global and Peripheral Interrupt Enable bits
    // Use the following macros to:

    // Enable the Global Interrupts
    INTERRUPT_GlobalInterruptEnable();

    // Enable the Peripheral Interrupts
    INTERRUPT_PeripheralInterruptEnable();

    // Disable the Global Interrupts
    //INTERRUPT_GlobalInterruptDisable();

    // Disable the Peripheral Interrupts
    //INTERRUPT_PeripheralInterruptDisable();

	unsigned char ch;
	uint8_t beat1=0, beat2=0;
	uint8_t cmbbuf_chars=0;
	int8_t erg;

	LED_Heart_SetHigh();

	init_time();
	init_sensors();
	init_rfid();
	init_stepper();
	INTERRUPT_GlobalInterruptEnable();

#define BEAT_LOW 85
#define BEAT_HIGH 12
    while (1)
    {
		__delay_ms(10);
		beat2++;
		if ((beat1 && beat2>=BEAT_HIGH) || (!beat1 && beat2>=BEAT_LOW))
		{	LED_Heart_Toggle();
			if (beat1++>=3) beat1=0;
			beat2=0;
		}
		if (testchar())
		{	ch=getchar();
			if (cmbbuf_chars<CMDBUFFER_LEN-1 &&
				((ch>='a' && ch<='z') || (ch>='A' && ch<='Z') || (ch>='0' && ch<='9') || ch=='-'))
				cmdbuffer[cmbbuf_chars++]=ch;
			else if (ch=='\n' || ch=='\r')
			{	if (cmbbuf_chars>0)
				{	erg=-1;
					cmdbuffer[cmbbuf_chars]='\0';

					ch=cmdbuffer[0];
					if (ch=='-' || (ch>='0' && ch<='9'))
					{	move_stepper(atoi(cmdbuffer));
						erg=0;
					}
					else
					{	uint8_t i;
						for (i=0; i<sizeof(cmds)/sizeof(struct cmd) && strcmp(cmdbuffer, cmds[i].cmd_txt); i++);
						if (i<sizeof(cmds)/sizeof(struct cmd))
							erg=cmds[i].cmd_fn();
					}
					if (erg==0) putstr("ok: ");
					if (erg<0) putstr("unkown command: ");
					if (erg<=0) putstr(cmdbuffer);
				}
				putstr("\r\n");
				cmbbuf_chars=0;
			}
		}
			
		service_time();
		service_sensors();
		service_rfid();
		service_io();
		CLRWDT();
    }
}
/**
 End of File
*/
