/*
 code for the stepper motor for the 4way flap clock switch
 and code to move the lock of the sureflap up and down by command

 $Id: stepper.c,v 1.4 2019/07/07 10:34:45 daeb Exp $
*/

/*

 Copyright (C) 2019 Bernhard Dubler ( klappe@daeublers.de )

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "mcc_generated_files/mcc.h"
#include "stepper.h"
#include "io.h"

#define STEPBIT1 4
#define STEPBIT2 3
#define STEPBIT3 6
#define STEPBIT4 7
unsigned char stepstates[]=
	{ (1<<STEPBIT1),
	  (1<<STEPBIT2) | (1<<STEPBIT1),
	  (1<<STEPBIT2),
	  (1<<STEPBIT3) | (1<<STEPBIT2),
	  (1<<STEPBIT3),
	  (1<<STEPBIT4) | (1<<STEPBIT3),
	  (1<<STEPBIT4),
	  (1<<STEPBIT1) | (1<<STEPBIT4),
	};
			
#define STEP_BITMASK ((1<<STEPBIT4) | (1<<STEPBIT3) | (1<<STEPBIT2) | (1<<STEPBIT1))

void move_riegel(uint8_t up)
{	if (up)
	{	MotUp_SetHigh();
		MotDown_SetLow();
	}
	else
	{	MotUp_SetLow();
		MotDown_SetHigh();
	}
	for (up=0; up<4; up++)
	{	MotEnable_SetHigh();
		__delay_ms(200);
		MotEnable_SetLow();
		__delay_ms(100);
		CLRWDT();
		service_io();
	}
}

void set_stepbits(unsigned char bits)
{
		do { LATC=(LATC&~(unsigned char)STEP_BITMASK) | (unsigned char)(bits&STEP_BITMASK); } while(0);
}

static int8_t stepper_state=0;

void move_stepper(int degrees)
{	int steps;
	steps=(long)degrees*512*8/360;

	steps=-steps;
	set_stepbits(stepstates[stepper_state]);
		__delay_ms(1);
	while (steps>0)
	{	stepper_state++;
		if (stepper_state>=sizeof(stepstates)) stepper_state=0;
		set_stepbits(stepstates[stepper_state]);
		steps--;
		if (!(steps&0xff))
		{	CLRWDT();
			service_io();
		}
		__delay_ms(1);
	}
	while (steps<0)
	{	stepper_state--;
		if (stepper_state<0) stepper_state=sizeof(stepstates)-1;
		set_stepbits(stepstates[stepper_state]);
		steps++;
		if (!(steps&0xff))
		{	CLRWDT();
			service_io();
		}
		__delay_ms(1);
	}
	set_stepbits(0);
}

void init_stepper(void)
{	move_stepper(0);
	MotUp_SetLow();
	MotDown_SetLow();
	MotEnable_SetLow();
}
